package dev.fyloz.trial.colorrecipesexplorer.core.services;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import dev.fyloz.trial.colorrecipesexplorer.ColorRecipesExplorerApplication;
import dev.fyloz.trial.colorrecipesexplorer.core.model.IModel;
import org.slf4j.Logger;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.lang.NonNull;

import java.util.List;
import java.util.Optional;

public abstract class GenericService<T extends IModel, R extends JpaRepository<T, Long>> implements IGenericService<T> {

    protected Logger logger = ColorRecipesExplorerApplication.LOGGER;

    protected R dao;

    public GenericService(R dao) {
        this.dao = dao;
    }

    @Override
    public Optional<T> getById(Long id) {
        return dao.findById(id);
    }

    @Override
    public List<T> getAll() {
        return dao.findAll();
    }

    @Override
    public Optional<T> save(T entity) {
        if (isValidForCreation(entity)) {
            return Optional.of(dao.save(entity));
        }

        return Optional.empty();
    }

    @Override
    public boolean saveAll(Iterable<T> entities) {
        if (entities == null) return false;

        for (T e : entities) {
            if (!save(e).isPresent()) {
                return false;
            }
        }

        dao.saveAll(entities);

        return true;
    }

    @Override
    public Optional<T> update(@NonNull T entity) {
        if (isValidForUpdate(entity)) {
            return Optional.of(dao.save(entity));
        }

        return Optional.empty();
    }

    @Override
    public boolean delete(T entity) {
        if (entity == null) return false;

        if (exists(entity)) {
            dao.delete(entity);
            return true;
        }

        return false;
    }

    @Override
    public boolean deleteAll(List<T> entities) {
        if (entities == null) return false;

        for (T entity : entities) {
            if (!exists(entity)) return false;
        }

        dao.deleteAll(entities);
        return true;
    }

    /**
     * Vérifie si une entité est valide pour la création dans la base de données.
     *
     * @param entity L'entité à vérifier.
     * @return Si l'entité est valide pour la création.
     */
    public boolean isValidForCreation(T entity) {
        return entity != null && !exists(entity);
    }

    /**
     * Vérifie si une entité est valide pour la création dans la base de données.
     *
     * @param entity L'entité à vérifier.
     * @return Si l'entité est valide pour l'édition.
     */
    public boolean isValidForUpdate(@NonNull T entity) {
        return exists(entity);
    }

    /**
     * Vérifie si une entité existe dans la base de données.
     *
     * @param entity L'entité à vérifier
     * @return Si l'entité existe.
     */
    @Override
    public boolean exists(T entity) {
        return entity != null && existsById(entity.getId());
    }

    @Override
    public boolean existsById(Long id) {
        return dao.existsById(id);
    }

    /**
     * Transforme un objet en Json.
     *
     * @param obj L'objet à transformer
     * @return L'objet sous forme de Json.
     */
    public String asJson(Object obj) {
        ObjectMapper objectMapper = new ObjectMapper();
        try {
            return objectMapper.writeValueAsString(obj);
        } catch (JsonProcessingException e) {
            logger.error("Une erreur est survenue lors de la transformation d'un objet en Json", e);
            return null;
        }
    }
}
