package dev.fyloz.trial.colorrecipesexplorer.web.controller.removers;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.REMOVER_RECIPE;
import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.REMOVER_RECIPE_SPECIFIC;

@Controller
public class RecipeRemoverController {

    private RecipeService recipeService;

    public RecipeRemoverController(RecipeService recipeService) {
        this.recipeService = recipeService;
    }

    /**
     * Affiche la liste de toutes les recettes.
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(REMOVER_RECIPE)
    public ModelAndView listRecipes(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(REMOVER_RECIPE)
                .addResponseData(ResponseDataType.RECIPE_MAP, recipeService.getRecipesByCompany())
                .build();
    }

    /**
     * Permet à l'utilisateur de supprimer une recette.
     * Cette méthode requiert l'identifiant de la recette dans l'URL.
     * <p>
     * La suppression échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - La recette n'existe pas
     * - Une erreur est survenue lors de la suppression dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - recipeCode: Contient la couleur de la recette
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param id L'identifiant de la recette
     * @return La page à afficher.
     */
    @PostMapping(REMOVER_RECIPE_SPECIFIC)
    public ModelAndView removeRecipe(@PathVariable Long id) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder("");

        Optional<Recipe> optionalRecipe = recipeService.getById(id);
        if (optionalRecipe.isEmpty()) {
            return listRecipes(
                    modelResponseBuilder
                            .addResponseCode(ResponseCode.RECIPE_NOT_FOUND, id)
                            .build());
        }

        Recipe recipe = optionalRecipe.get();
        if (!recipeService.deleteRecipe(recipe)) {
            return listRecipes(
                    modelResponseBuilder
                            .addResponseCode(ResponseCode.ERROR_SAVING)
                            .build());
        }

        modelResponseBuilder.addResponseCode(ResponseCode.SUCCESS_DELETING_RECIPE, recipe.getName());
        return listRecipes(modelResponseBuilder.build());
    }
}
