package dev.fyloz.trial.colorrecipesexplorer.core.services;

import dev.fyloz.trial.colorrecipesexplorer.core.model.Company;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.CompanyService;
import dev.fyloz.trial.colorrecipesexplorer.dao.CompanyDao;
import dev.fyloz.trial.colorrecipesexplorer.dao.RecipeDao;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static org.junit.Assert.*;
import static org.mockito.Mockito.*;

public class CompanyServiceTests {

    @Mock
    private CompanyDao companyDao;

    @Mock
    private RecipeDao recipeDao;

    @Spy
    @InjectMocks
    private CompanyService companyService;

    private Company testCompany;
    private Company testCompany2;
    private List<Company> testCompaniesList;

    @Before
    public void init() {
        MockitoAnnotations.initMocks(this);

        testCompany = new Company("Test Company");
        testCompany.setCompanyId(0);

        testCompany2 = new Company("Test Company 2");
        testCompany.setCompanyID(1);

        testCompaniesList = new ArrayList<>();
        testCompaniesList.add(testCompany);
        testCompaniesList.add(testCompany2);
    }

    @Test
    public void isValidForCreationTestShouldBeTrue() {
        // Arrange
        when(companyService.existsByName(testCompany.getName())).thenReturn(false);
        when(companyService.exists(testCompany)).thenReturn(false);

        // Act
        boolean actual = companyService.isValidForCreation(testCompany);

        // Assert
        assertTrue(actual);
    }

    @Test
    public void isValidForCreationTestShouldBeFalse() {
        // Arrange
        when(companyService.existsByName(testCompany.getName())).thenReturn(true);
        when(companyService.exists(testCompany)).thenReturn(true);

        // Act
        boolean actual = companyService.isValidForCreation(testCompany);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void isValidForCreationTestExists() {
        // Arrange
        when(companyService.existsByName(testCompany.getName())).thenReturn(false);
        when(companyService.exists(testCompany)).thenReturn(true);

        // Act
        boolean actual = companyService.isValidForCreation(testCompany);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void isValidForCreationTestNameExists() {
        // Arrange
        when(companyService.existsByName(testCompany.getName())).thenReturn(true);
        when(companyService.exists(testCompany)).thenReturn(false);

        // Act
        boolean actual = companyService.isValidForCreation(testCompany);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void isValidForCreationTestNull() {
        // Act
        boolean actual = companyService.isValidForCreation(null);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void existsByNameTestShouldBeTrue() {
        // Arrange
        when(companyDao.existsByCompanyName(testCompany.getName())).thenReturn(true);

        // Act
        boolean actual = companyService.existsByName(testCompany.getName());

        // Assert
        assertTrue(actual);
    }

    @Test
    public void existsByNameTestShouldBeFalse() {
        // Arrange
        when(companyDao.existsByCompanyName(testCompany.getName())).thenReturn(false);

        // Act
        boolean actual = companyService.existsByName(testCompany.getName());

        // Assert
        assertFalse(actual);
    }

    @Test
    public void existsByNameTestNull() {
        // Act
        boolean actual = companyService.existsByName(null);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void isLinkedToRecipesTestShouldBeTrue() {
        // Arrange
        List<Recipe> recipesList = new ArrayList<>();
        recipesList.add(new Recipe());

        when(recipeDao.findAllByCompany(testCompany)).thenReturn(recipesList);

        // Act
        boolean actual = companyService.isLinkedToRecipes(testCompany);

        // Assert
        assertTrue(actual);
    }

    @Test
    public void isLinkedToRecipesTestShouldBeFalse() {
        // Arrange
        when(recipeDao.findAllByCompany(testCompany)).thenReturn(new ArrayList<>());

        // Act
        boolean actual = companyService.isLinkedToRecipes(testCompany);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void isLinkedToRecipesTestNull() {
        // Act
        boolean actual = companyService.isLinkedToRecipes(null);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void deleteIfNotLinkedTestShouldBeTrue() {
        // Arrange
        doReturn(true).when(companyService).delete(testCompany);
        doReturn(false).when(companyService).isLinkedToRecipes(testCompany);

        // Act
        boolean actual = companyService.deleteIfNotLinked(testCompany);

        // Assert
        assertTrue(actual);
    }

    @Test
    public void deleteIfNotLinkedTestShouldBeFalse() {
        // Arrange
        doReturn(true).when(companyService).delete(testCompany);
        doReturn(true).when(companyService).isLinkedToRecipes(testCompany);

        // Act
        boolean actual = companyService.deleteIfNotLinked(testCompany);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void deleteIfNotLinkedTestNull() {
        // Arrange
        doReturn(true).when(companyService).delete(testCompany);

        // Act
        boolean actual = companyService.deleteIfNotLinked(null);

        // Assert
        assertFalse(actual);
    }

    // Test les méthodes génériques
    @Test
    public void getByIDTest() {
        // Arrange
        doReturn(Optional.of(testCompany)).when(companyDao).findById(testCompany.getCompanyID());

        // Act
        Optional<Company> actual = companyService.getByID(testCompany.getCompanyID());

        // Assert
        assertTrue(actual.isPresent());
        assertEquals(testCompany, actual.get());
    }

    @Test
    public void getByIDTestShouldBeNull() {
        // Arrange
        doReturn(Optional.empty()).when(companyDao).findById(testCompany.getCompanyID());

        // Act
        Optional<Company> actual = companyService.getByID(testCompany.getCompanyID());

        // Arrange
        assertFalse(actual.isPresent());
    }

    @Test
    public void getAllTest() {
        // Arrange
        List<Company> companies = new ArrayList<>();
        companies.add(testCompany);

        doReturn(companies).when(companyDao).findAll();

        // Act
        List<Company> actual = companyService.getAll();

        // Assert
        assertTrue(actual.contains(testCompany));
    }

    @Test
    public void getAllTestShouldBeEmpty() {
        // Arrange
        doReturn(new ArrayList<>()).when(companyDao).findAll();

        // Act
        List<Company> actual = companyService.getAll();

        // Assert
        assertTrue(actual.isEmpty());
    }

    @Test
    public void saveTest() {
        // Arrange
        doReturn(true).when(companyService).isValidForCreation(testCompany);
        doReturn(testCompany).when(companyDao).save(testCompany);

        // Act
        Optional<Company> actual = companyService.save(testCompany);

        // Assert
        assertTrue(actual.isPresent());
        assertEquals(testCompany, actual.get());
    }

    @Test
    public void saveTestInvalidCompany() {
        // Arrange
        doReturn(false).when(companyService).isValidForCreation(testCompany);
        doReturn(testCompany).when(companyDao).save(testCompany);

        // Act
        Optional<Company> actual = companyService.save(testCompany);

        // Assert
        assertFalse(actual.isPresent());
    }

    @Test
    public void saveTestNull() {
        // Arrange
        doReturn(false).when(companyService).isValidForCreation(testCompany);
        doReturn(testCompany).when(companyDao).save(testCompany);

        // Act
        Optional<Company> actual = companyService.save(null);

        // Assert
        assertFalse(actual.isPresent());
    }

    @Test
    public void saveAllTestShouldBeTrue() {
        // Arrange
        doAnswer(i -> Optional.of(i.getArguments()[0])).when(companyService).save(any(Company.class));

        // Act
        boolean actual = companyService.saveAll(testCompaniesList);

        // Assert
        assertTrue(actual);
    }

    @Test
    public void saveAllTestShouldBeFalse() {
        // Arrange
        doAnswer(i -> Optional.empty()).when(companyService).save(any(Company.class));

        // Act
        boolean actual = companyService.saveAll(testCompaniesList);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void saveAllTestOneSaveFalse() {
        // Arrange
        doReturn(Optional.of(testCompany)).when(companyService).save(testCompany);
        doReturn(Optional.empty()).when(companyService).save(testCompany2);

        // Act
        boolean actual = companyService.saveAll(testCompaniesList);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void saveAllTestNull() {
        // Arrange
        doAnswer(i -> Optional.empty()).when(companyService).save(any(Company.class));

        // Act
        boolean actual = companyService.saveAll(null);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void updateTest() {
        // Arrange
        doReturn(true).when(companyService).isValidForUpdate(testCompany);
        doReturn(testCompany).when(companyDao).save(testCompany);

        // Act
        Optional<Company> actual = companyService.update(testCompany);

        // Assert
        assertTrue(actual.isPresent());
        assertEquals(testCompany, actual.get());
    }

    @Test
    public void updateTestInvalidCompany() {
        // Arrange
        doReturn(false).when(companyService).isValidForUpdate(testCompany);
        doReturn(testCompany).when(companyDao).save(testCompany);

        // Act
        Optional<Company> actual = companyService.update(testCompany);

        // Assert
        assertFalse(actual.isPresent());
    }

    @Test
    public void updateTestNull() {
        // Arrange
        doReturn(false).when(companyService).isValidForUpdate(testCompany);
        doReturn(testCompany).when(companyDao).save(testCompany);

        // Act
        Optional<Company> actual = companyService.update(null);

        // Assert
        assertFalse(actual.isPresent());
    }

    @Test
    public void deleteTestShouldBeTrue() {
        // Arrange
        doReturn(true).when(companyService).exists(testCompany);

        // Act
        boolean actual = companyService.delete(testCompany);

        // Assert
        assertTrue(actual);
    }

    @Test
    public void deleteTestShouldBeFalse() {
        // Arrange
        doReturn(false).when(companyService).exists(testCompany);

        // Act
        boolean actual = companyService.delete(testCompany);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void deleteTestNull() {
        // Arrange
        doReturn(true).when(companyService).exists(testCompany);

        // Act
        boolean actual = companyService.delete(null);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void deleteAllTestShouldBeTrue() {
        // Arrange
        doReturn(true).when(companyService).exists(any());

        // Act
        boolean actual = companyService.deleteAll(testCompaniesList);

        // Assert
        assertTrue(actual);
    }

    @Test
    public void deleteAllTestShouldBeFalse() {
        // Arrange
        doReturn(false).when(companyService).exists(any());

        // Act
        boolean actual = companyService.deleteAll(testCompaniesList);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void deleteAllTestNull() {
        // Arrange
        doReturn(false).when(companyService).exists(any());

        // Act
        boolean actual = companyService.deleteAll(null);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void existsTestShouldBeTrue() {
        // Arrange
        doReturn(true).when(companyService).existsById(testCompany.getCompanyID());

        // Act
        boolean actual = companyService.exists(testCompany);

        // Assert
        assertTrue(actual);
    }

    @Test
    public void existsTestShouldBeFalse() {
        // Arrange
        doReturn(false).when(companyService).existsById(testCompany.getCompanyID());

        // Act
        boolean actual = companyService.exists(testCompany);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void existsTestNull() {
        // Arrange
        doReturn(true).when(companyService).existsById(testCompany.getCompanyID());

        // Act
        boolean actual = companyService.exists(null);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void existsByIdTestShouldBeTrue() {
        // Arrange
        doReturn(true).when(companyDao).existsById(testCompany.getCompanyID());

        // Act
        boolean actual = companyService.exists(testCompany);

        // Assert
        assertTrue(actual);
    }

    @Test
    public void existsByIdTestShouldBeFalse() {
        // Arrange
        doReturn(false).when(companyDao).existsById(testCompany.getCompanyID());

        // Act
        boolean actual = companyService.exists(testCompany);

        // Assert
        assertFalse(actual);
    }

    @Test
    public void existsByIdTestNull() {
        // Arrange
        doReturn(true).when(companyDao).existsById(testCompany.getCompanyID());

        // Act
        boolean actual = companyService.exists(null);

        // Assert
        assertFalse(actual);
    }
}
