package dev.fyloz.trial.colorrecipesexplorer.core.services;

import dev.fyloz.trial.colorrecipesexplorer.core.model.Mix;
import dev.fyloz.trial.colorrecipesexplorer.core.model.RecipeStep;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Company;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.CompanyService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MixService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.StepService;
import dev.fyloz.trial.colorrecipesexplorer.dao.RecipeDao;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

import java.util.*;

import static junit.framework.TestCase.assertTrue;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doReturn;

public class RecipeServiceTests {

    @Mock
    private CompanyService companyService;

    @Mock
    private StepService stepService;

    @Mock
    private MixService mixService;

    @Mock
    private RecipeDao recipeDao;

    @Spy
    @InjectMocks
    private RecipeService recipeService;

    @Mock
    private Recipe testRecipe;
    private Company testCompany;
    private List<Mix> testMixesList;

    @Before
    public void init() {
        MockitoAnnotations.initMocks(this);

        testCompany = new Company();
        testCompany.setCompanyID(0);
        testCompany.setCompanyName("Test Company");

        testRecipe = new Recipe();
        testRecipe.setRecipeID(Integer.MAX_VALUE);
        testRecipe.setName("Test Recipe");
        testRecipe.setCompany(testCompany);

        Mix mix1 = new Mix();
        mix1.setMixID(5);
        Mix mix2 = new Mix();
        mix1.setMixID(1);
        testMixesList = new ArrayList<>();
        testMixesList.add(mix1);
        testMixesList.add(mix2);
    }

    @Test
    public void getByCompanyTest() {
        List<Recipe> testRecipesList = new ArrayList<>();
        testRecipesList.add(testRecipe);

        doReturn(testRecipesList).when(recipeDao).findAllByCompany(testCompany);

        List<Recipe> actual = recipeService.getByCompany(testCompany);
        assertFalse(actual.isEmpty());
    }

    @Test
    public void getByCompanyTestShouldBeEmpty() {
        doReturn(new ArrayList<>()).when(recipeDao).findAllByCompany(testCompany);

        List<Recipe> actual = recipeService.getByCompany(testCompany);
        assertTrue(actual.isEmpty());
    }

    @Test
    public void getByCompanyTestNull() {
        doReturn(new ArrayList<>()).when(recipeDao).findAllByCompany(any());

        List<Recipe> actual = recipeService.getByCompany(null);
        assertTrue(actual.isEmpty());
    }

    @Test
    public void deleteRecipeTestShouldBeTrue() {
        doReturn(true).when(recipeService).delete(testRecipe);
        doReturn(new ArrayList<>()).when(recipeService).getImageFiles(testRecipe);

        boolean actual = recipeService.deleteRecipe(testRecipe);
        assertTrue(actual);
    }

    @Test
    public void deleteRecipeTestShouldBeFalse() {
        doReturn(false).when(recipeService).delete(testRecipe);

        boolean actual = recipeService.deleteRecipe(testRecipe);
        assertFalse(actual);
    }

    @Test
    public void deleteRecipeTestNull() {
        doReturn(false).when(recipeService).delete(testRecipe);

        boolean actual = recipeService.deleteRecipe(null);
        assertFalse(actual);
    }

//    @Test
//    public void getSortedMixesTest() {
////        doReturn(testMixesList).when(testRecipe).getMixes();
//        testRecipe.setRecipeMixes(testMixesList);
//
//        List<Mix> actual = recipeService.getSortedMixes(testRecipe);
//        assertFalse(actual.isEmpty());
//        assertEquals(testMixesList.get(1).getId(), actual.get(0).getId());
//        assertEquals(testMixesList.get(0).getId(), actual.get(1).getId());
//    }
//
//    @Test
//    public void getSortedMixesTestShouldBeEmpty() {
//        testRecipe.setRecipeMixes(testMixesList);
//
//        List<Mix> actual = recipeService.getSortedMixes(testRecipe);
//        assertTrue(actual.isEmpty());
//    }
//
//    @Test
//    public void getSortedMixesTestNull() {
//        doReturn(new ArrayList<>()).when(testRecipe).getMixes();
//
//        List<Mix> actual = recipeService.getSortedMixes(null);
//        assertTrue(actual.isEmpty());
//    }

    @Test
    public void convertAndCreateStepsTest() {
        MultiValueMap<String, Object> multiValueMap = new LinkedMultiValueMap<>();
        List<String> testSteps = new LinkedList<>();
        testSteps.add("step 1");
        testSteps.add("step 2");
        testSteps.add("step 3");

        multiValueMap.add("step", testSteps);

        doReturn(true).when(stepService).deleteAll(any());
        doReturn(Optional.of(testRecipe)).when(recipeService).save(any());

        Optional<Recipe> actual = recipeService.convertAndCreateSteps(testRecipe, multiValueMap);
        assertTrue(actual.isPresent());
    }

    @Test
    public void convertAndCreateStepsTestWithPreviousSteps() {
        MultiValueMap<String, Object> multiValueMap = new LinkedMultiValueMap<>();
        List<String> testSteps = new LinkedList<>();
        testSteps.add("step 1");
        testSteps.add("step 2");
        testSteps.add("step 3");

        multiValueMap.add("step", testSteps);

        List<RecipeStep> previousSteps = new ArrayList<>();
        previousSteps.add(new RecipeStep(testRecipe, "previous 1"));
        previousSteps.add(new RecipeStep(testRecipe, "previous 2"));
        testRecipe.setRecipeSteps(previousSteps);

        doReturn(true).when(stepService).deleteAll(any());
        doReturn(Optional.of(testRecipe)).when(recipeService).save(any());

        Optional<Recipe> actual = recipeService.convertAndCreateSteps(testRecipe, multiValueMap);
        assertTrue(actual.isPresent());
    }

    @Test
    public void convertAndCreateStepsTestShouldBeEmpty() {
        MultiValueMap<String, Object> multiValueMap = new LinkedMultiValueMap<>();
        List<String> testSteps = new LinkedList<>();
        testSteps.add("step 1");
        testSteps.add("step 2");
        testSteps.add("step 3");

        multiValueMap.add("step", testSteps);

        List<RecipeStep> previousSteps = new ArrayList<>();
        previousSteps.add(new RecipeStep(testRecipe, "previous 1"));
        previousSteps.add(new RecipeStep(testRecipe, "previous 2"));
        testRecipe.setRecipeSteps(previousSteps);

        doReturn(false).when(stepService).deleteAll(any());
        doReturn(Optional.empty()).when(recipeService).save(any());

        Optional<Recipe> actual = recipeService.convertAndCreateSteps(testRecipe, multiValueMap);
        assertFalse(actual.isPresent());
    }

    @Test
    public void convertAndCreateStepsTestNull() {
        doReturn(false).when(stepService).deleteAll(any());
        doReturn(Optional.empty()).when(recipeService).save(any());

        Optional<Recipe> actual = recipeService.convertAndCreateSteps(null, null);
        assertFalse(actual.isPresent());
    }

}
